"""Module for modifying the axis properties of plots."""

import matplotlib.pyplot as plt
import numpy as np
from matplotlib import ticker


def change_log_axis_base(axes: plt.Axes, which: str, base: float = 10) -> plt.Axes:
    """Change the tick formatter to not use powers 0 and 1 in logarithmic plots.

    Change the logarithmic axes `10^0 -> 1` and `10^1 -> 10` (or the given base), i.e.
    without power, otherwise use the base to some power. For more robust and less error
    prone results, the plotting type is also re-set with the same base ('loglog',
    'semilogx' and 'semilogy').

    Modified from: https://tinyurl.com/log-tick-formatting

    Parameters
    ----------
    axes: plt.Axes
        An axes object
    which: str
        Whether to update both x and y axis, or just one of them.
    base: float
        The base of the logarithm. Defaults to base 10 (same as loglog, etc.)

    Returns
    -------
    plt.Axes
        The updated axes object.

    Raises
    ------
    ValueError
        If the axes given in `which` is not `x`, `y` or `both`.
    """
    if which == "both":
        axs, pltype = ["xaxis", "yaxis"], "loglog"
    elif which == "x":
        axs, pltype = ["xaxis"], "semilogx"
    elif which == "y":
        axs, pltype = ["yaxis"], "semilogy"
    else:
        raise ValueError(
            "No valid axis found. 'which' must be either of 'both', 'x' or 'y'."
        )
    getattr(axes, pltype)(base=base)
    for ax in axs:
        f = getattr(axes, ax)
        f.set_major_formatter(
            ticker.FuncFormatter(
                lambda x, _: r"${:g}$".format(x)
                if np.log(x) / np.log(base) in [0, 1]
                else r"$"
                + str(base)
                + "^{"
                + "{:g}".format(np.log(x) / np.log(base))
                + r"}$"
            )
        )
    return axes



#!/usr/bin/python
# -*- Encoding: UTF-8 -*-

from typing import List
import numpy as np
import matplotlib as mpl

"""
Definitions for figures and plots.
Use the set_rcparams_.... routines to get a sane default configuration
for production quality plots.
Some guidelines when creating plots
As of matplotlib2.0, use the vega 3 color palette
https://github.com/vega/vega/wiki/Scales#scale-range-literals
For density scans, use the following plot styles
Density    Symbol    Color
High         ^         C0 #1f77b4
 ^           s         C1 #ff7f0e
 |           o         C2 #2ca02c
 v           d         C3 #d62728
Low          v         C4 #d9467bd
Color are robust on both, screen, print, projector.
"""

# For 5 symbols in a plot
symbol_list_5 = ["^", "s", "o", "d", "v"]

# For 4 symbols in a plot and below, avoid red and green (colorblindness) if possible
symbol_list_4 = ["^", "s", "d", "v"]

# 3 symbols in a plot
symbol_list_3 = ["^", "o", "v"]

# 2 symbols in a plot
symbol_list_2 = ["^", "v"]

# 1 symbol
symbol_list_1 = ["o"]


def set_rcparams_dynamo(
    myParams: mpl.RcParams, num_cols: int = 1, ls: str = "thin"
) -> List[float]:
    """Half column width figures for revtex.

    See http://publishing.aip.org/authors/preparing-graphics

    Parameters
    ----------
    myParams: matplotlib.RcParams
        The parameters from matplotlib
    num_cols: int
        Number of columns the figure spans.
        1 column  = 3.37" by 2.08"
        2 columns = 6.69" by 4.13"
    ls: str
        Either 'thick' or 'thin'. Defaults to 'thin'
    fonts: str
        Either 'small' or 'large'.

    Returns
    -------
    List[float]
        The size of the axes.

    Raises
    ------
    ValueError
        If the number of columns is not 1 or 2.
    """

    golden_ratio = 0.5 * (1.0 + np.sqrt(5.0))
    fig_dpi = 300.0
    fontsize = 8

    linewidth = 0.75
    if ls == "thick":
        linewidth *= 2

    # Define axis size to be used
    if num_cols == 1:
        ax_x0, ax_y0 = 0.2, 0.2
        axes_size = [ax_x0, ax_y0, 0.95 - ax_x0, 0.95 - ax_y0]
    elif num_cols == 2:
        ax_x0, ax_y0 = 0.1, 0.2
        axes_size = [ax_x0, ax_y0, 0.975 - ax_x0, 0.95 - ax_y0]
    else:
        raise ValueError("num_cols must be 1 or 2")

    # Figure size in inch
    fig_width_in = num_cols * 3.37
    fig_height_in = 3.37 / golden_ratio

    # Figure size and dpi
    myParams["figure.dpi"] = fig_dpi
    myParams["figure.figsize"] = [fig_width_in, fig_height_in]
    myParams["savefig.dpi"] = fig_dpi

    # Figure legend
    myParams["legend.framealpha"] = 1.0
    myParams["legend.fancybox"] = False
    myParams["legend.edgecolor"] = "k"
    myParams["patch.linewidth"] = 0.5  # For legend box borders
    myParams["legend.handlelength"] = 1.45  # Show nice, even
    # numbers for different line styles

    # Font and text
    myParams["text.usetex"] = True
    myParams["pdf.fonttype"] = 42
    myParams["font.family"] = "Times"
    myParams["font.size"] = fontsize
    myParams["axes.labelsize"] = fontsize
    myParams["legend.fontsize"] = fontsize

    # Line size and marker size
    myParams["lines.markersize"] = 3.0 * linewidth
    myParams["lines.linewidth"] = linewidth

    # Axes thickness
    myParams["axes.linewidth"] = 0.5
    # Enable minor ticks
    myParams["ytick.minor.visible"] = True
    myParams["xtick.minor.visible"] = True
    # Default ticks on both sides of the axes
    myParams["xtick.top"] = True
    myParams["xtick.bottom"] = True
    myParams["ytick.left"] = True
    myParams["ytick.right"] = True
    # All ticks point inward
    myParams["xtick.direction"] = "in"
    myParams["ytick.direction"] = "in"

    return axes_size


def set_rcparams_article_thickline(myParams: mpl.RcParams) -> None:
    """
    One 8cm column for the TCV paper, thicker lines for visibility
    """
    set_rcparams_dynamo(myParams, ls="thick")

    # Re-sets:
    myParams["lines.markersize"] = 2


def set_rcparams_poster(myParams: mpl.RcParams) -> None:
    """
    Make 12.5cm wide figures used in posters
    """
    set_rcparams_dynamo(myParams, ls="thick")
    golden_ratio = 0.5 * (1.0 + np.sqrt(5.0))
    fontsize = 8

    # Re-sets: (in alphabetical order)
    fig_width_in = 4.92
    fig_height_in = fig_width_in / golden_ratio
    myParams["axes.labelsize"] = fontsize * 2
    myParams["axes.linewidth"] = 2
    myParams["figure.figsize"] = [fig_width_in, fig_height_in]
    myParams["font.size"] = fontsize * 2
    myParams["legend.fontsize"] = 12
    myParams["lines.linewidth"] = 2
    myParams["patch.linewidth"] = 2  # For legend box borders

    # New settings:
    myParams["savefig.dpi"] = 300
    myParams["xtick.major.width"] = 2
    myParams["xtick.minor.width"] = 1
    myParams["ytick.major.width"] = 2
    myParams["ytick.minor.width"] = 1


def set_rcparams_talk(myParams: mpl.RcParams) -> mpl.RcParams:
    """
    Matplotlib configuration for talk graphics.
    Use for 16:9 aspect ratio in beamer slides
    Slides are 16cm * 9cm, figure is 7.5cm wide
    """
    set_rcparams_dynamo(myParams, ls="thick")
    golden_ratio = 0.5 * (1.0 + np.sqrt(5.0))

    # Re-sets: (in alphabetical order)
    fig_width_in = 6
    fig_height_in = fig_width_in / golden_ratio
    myParams["axes.linewidth"] = 1.0
    myParams["figure.figsize"] = [fig_width_in, fig_height_in]
    myParams["font.size"] = 16
    myParams["legend.fontsize"] = 12

    # New settings:
    myParams["text.latex.unicode"] = True

    return myParams


# End of file figure_defs.py



import numpy as np
from matplotlib.colors import LinearSegmentedColormap
import matplotlib.pyplot as plt
from matplotlib import cm, colors
from typing import List


def make_color_swatch(color_list: list) -> LinearSegmentedColormap:
    """
    Generate plot of the desired color swatches. This is useful if you want to see what the swatches look like.
    This function is used in generate_hex_colors() where the plot of the swatches will show by default if used in Jupyter Notebook.

    Parameters
    ----------
    color_list: list
        List of colors from color map

    Returns
    -------
    color_swatch: LinearSegmentedColormap
        A plot of the color swatches
    """

    color_swatch = LinearSegmentedColormap.from_list(
        "my_list", [colors.to_rgb(x) for x in color_list]
    )

    plt.figure(figsize=(5, 3))
    plt.imshow(
        [list(np.arange(0, len(color_list), 1))],
        interpolation="nearest",
        origin="lower",
        cmap=color_swatch,
    )
    plt.xticks([])
    plt.yticks([])

    return color_swatch


def generate_hex_colors(
    number_data_points: int,
    color_map: str,
    show_swatch: bool = True,
    ascending: bool = True,
) -> List[str]:
    """
    Function to generate colors picked out of the matplolib color maps as hex numbers.
    If using this function in Jupyter Notebook, by default since show_swatch = True, a plot of the swatches will automatically appear.

    Parameters
    ----------
    number_data_points: int
        The number of data points
    color_map: str
        The name of the colour map from matplotlib. Grayscale friendly colors are:
        'viridis', 'plasma', 'inferno', 'magma', 'cubehelix', 'cividis'
        See more here: https://matplotlib.org/stable/tutorials/colors/colormaps.html
    show_swatch: bool = True
        Show the color swatches based on the number of data points. 'True' means that a plot will show containing the swatches.
    ascending: bool = True
        Show the color swatches and list in ascending order, from light to dark. Set to True by default.

    Returns
    -------
    color_list: List[str]
        List of hex numbers from the desired color map. Simply do print(color_list) to get this list.

    """

    cmap = cm.get_cmap(color_map, number_data_points)

    color_list = []

    for i in range(cmap.N):
        rgba = cmap(i)

        # rgb2hex accepts rgb or rgba
        color_list.append(colors.rgb2hex(rgba))

    # return light to darkest if ascending
    if ascending:
        if show_swatch:
            make_color_swatch(color_list[::-1])
        return color_list[::-1]
    else:
        if show_swatch:
            make_color_swatch(color_list)
        return color_list
